# -*- coding: utf-8 -*-

"""Command line interface for trialbureautools. Exposes functions from the 'tools' module. This module should not
contain important functionality but rather just define the command line interface.

docstrings in this module are displayed by the click CLI so should be user-centered."""

import sys
from pathlib import Path

import click

from trialbureautools.tools import set_folder_rights, PERMISSIONS, ToolsException, IDISOutputFolder


@click.group()
def cli():
    """Trial bureau scripts"""
    pass


@click.command(short_help="Set folder permissions")
@click.argument('folder', type=click.Path(exists=True, resolve_path=True))
@click.argument('username', type=str)
@click.argument('permission_name', type=click.Choice(list(PERMISSIONS.keys())))
def set_folder_permissions(folder, username, permission_name):
    """Set permissions for given windows folder and windows user

    Example:

        set_folder_permissions C:\folder1 z123456 full_access\b

        sets the permissions for C:\folder1 so that user 'z123456' has full access

    """
    try:
        set_folder_rights(folder=folder, username=username, permission=PERMISSIONS[permission_name])
        click.echo(f"Set folder '{folder}' permissions to {permission_name}")
    except ToolsException as e:
        click.echo(f'Error: { str(e) }')


@click.command(short_help="Create an IDIS output folder in the given folder")
@click.argument('base_folder', type=click.Path(exists=True, resolve_path=True))
@click.argument('z_number', type=str)
def create_idis_output_folder(base_folder, z_number):
    """Create an IDIS output folder in the given folder with permissions to read and delete but not write

        Example:

        create_idis_output_folder . z123456

        creates a folder 'z123456' in the current directory, and sets permissions
    """
    folder = IDISOutputFolder(base_folder=Path(base_folder), z_number=z_number)
    if click.confirm(f'This will create an IDIS output folder in "{folder.path}"'):
        try:
            folder.initialize()
            click.echo(f'Done')
        except ToolsException as e:
            click.echo(f'Error: { str(e) }')
    else:
        click.echo("cancelled")


cli.add_command(set_folder_permissions)
cli.add_command(create_idis_output_folder)

if __name__ == "__main__":
    sys.exit(cli())  # pragma: no cover



# -*- coding: utf-8 -*-

""" Python functions that could be of use for the trial bureau.

Docstrings in this module are for programmers, so can be as detailed is needed """
import os

from icaclswrap.foldertool import WinFolderPermissionTool, ACLToolException

from icaclswrap.rights import FULL_ACCESS, READ_DELETE

PERMISSIONS = {'full_access': FULL_ACCESS,
               'read_delete': READ_DELETE}


def set_folder_rights(folder, username, permission):
    """Set permissions from given

    Parameters
    ----------
    folder: path
        path to set permissions for. Can be relative or absolute
    username: str
        username to set permissions for
    permission_name: str
        choice of any of trialbureautools.permissions.PERMISSIONS

    Raises
    ------
    ToolsException
        when anything goes wrong with setting folder rights

    """

    tool = WinFolderPermissionTool()
    try:
        tool.set_rights(path=folder, username=username, rights_collection=permission)
    except ACLToolException as e:
        raise ToolsException(str(e))


class IDISOutputFolder:

    def __init__(self, base_folder, z_number):
        """Create z-number folder and set permissions

        Parameters
        ----------
        base_folder: Path
            create folder in this path
        z_number: str
            valid radboudumc z-number, like z123456

        Raises
        ------
        ToolsException
            when anything goes wrong with setting folder rights

        """

        self.z_number = z_number
        self.path = str(base_folder / z_number)
        self.initialized = False

    def initialize(self):
        try:
            os.mkdir(self.path)
        except FileExistsError as e:
            raise ToolsException(str(e))
        set_folder_rights(folder=self.path, username=self.z_number, permission=READ_DELETE)


class ToolsException(Exception):
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `trialbureautools` package."""
import pytest

from unittest.mock import patch
from click.testing import CliRunner

from icaclswrap.foldertool import WinFolderPermissionTool
from trialbureautools.cli import set_folder_permissions, create_idis_output_folder

from tests import BASE_PATH
from trialbureautools.tools import ToolsException, IDISOutputFolder


@pytest.fixture()
def cli_runner():
    return CliRunner()


@pytest.mark.parametrize(
    "path_in, username_in, permission_in, expected_output",
    [
        (
            ".",
            "crazypermission",
            "test",
            "invalid choice",
        ),  # invalid choice for permission
        (
            r"\non_existent\folder",
            "testuser",
            "full_access",
            "does not exist",
        ),  # calling on non-existent path
    ],
)
def test_set_folder_permissions_faulty_parameter_responses(
    path_in, permission_in, username_in, expected_output, cli_runner
):
    """ Commands that are wrong somehow should be handled properly by click
    """
    with patch.object(WinFolderPermissionTool, "set_rights") as mock_set_rights:
        result = cli_runner.invoke(
            set_folder_permissions, [path_in, username_in, permission_in]
        )
    assert not mock_set_rights.called
    assert result.exit_code == 2
    assert expected_output in result.output


@pytest.mark.parametrize(
    "path_in, username_in, permission_in",
    [
        (".", "testuser", "full_access"),  # relative path
        (str(BASE_PATH), "testuser", "full_access"),  # absolute path ,
        (str(BASE_PATH), "testuser", "read_delete"),  # other permission ,
    ],
)
def test_set_folder_permissions_success(
    path_in, permission_in, username_in, cli_runner
):
    """ Calling 'set_folder_permissions' an a few ways that should work'
    """
    with patch(
        "trialbureautools.tools.WinFolderPermissionTool.set_rights", autospec=True
    ) as mock_set_rights:
        result = cli_runner.invoke(
            set_folder_permissions, [path_in, username_in, permission_in]
        )
    assert "Set folder" in result.output
    assert mock_set_rights.called
    assert result.exit_code == 0


def test_tools_exception_handling(cli_runner):
    """ Tools might raise exceptions. These should be caught and displayed simply"""

    with patch(
        "trialbureautools.tools.WinFolderPermissionTool.set_rights", autospec=True
    ) as mock_set_rights:
        mock_set_rights.side_effect = ToolsException("Something went very wrong")
        result = cli_runner.invoke(set_folder_permissions, [".", "user", "full_access"])
        assert "Something went very wrong" in result.output
        assert result.exit_code == 0


def test_create_CLI_function(cli_runner):
    with patch(
        "trialbureautools.tools.WinFolderPermissionTool.set_rights", autospec=True
    ) as mock_set_rights:
        with cli_runner.isolated_filesystem():
            result = cli_runner.invoke(
                create_idis_output_folder, [".", "z123456"], input="yes"
            )
            assert not result.exception
        assert mock_set_rights.called
